"""Stub file for reflex_enterprise/components/mantine/tags_input.py"""

# ------------------- DO NOT EDIT ----------------------
# This file was generated by `reflex/utils/pyi_generator.py`!
# ------------------------------------------------------
from typing import Any, Callable, Literal, Mapping, Optional, Sequence

from reflex.components.component import Component
from reflex.components.core.breakpoints import Breakpoints
from reflex.event import EventType, PointerEventInfo
from reflex.vars.base import Var

from reflex_enterprise.components.mantine.base import BaseMantineInput

class TagsInput(BaseMantineInput):
    @classmethod
    def create(
        cls,
        *children,
        accept_value_on_blur: Var[bool] | bool | None = None,
        allow_duplicates: Var[bool] | bool | None = None,
        clear_button_props: Var[dict] | dict | None = None,
        combobox_props: Var[dict] | dict | None = None,
        data: Var[list[str]] | list[str] | None = None,
        default_dropdown_opened: Var[bool] | bool | None = None,
        default_search_value: Var[str] | str | None = None,
        default_value: Var[list[str]] | list[str] | None = None,
        description: Component | Var[Component | str] | str | None = None,
        description_props: Var[dict] | dict | None = None,
        disabled: Var[bool] | bool | None = None,
        dropdown_opened: Var[bool] | bool | None = None,
        error: Component | Var[Component | bool | str] | bool | str | None = None,
        error_props: Var[dict] | dict | None = None,
        filter: Callable | Var[Callable] | None = None,
        hidden_input_props: Var[dict] | dict | None = None,
        hidden_input_values_divider: Var[str] | str | None = None,
        input_container: Component | Var[Component] | None = None,
        input_size: Var[str] | str | None = None,
        input_wrapper_order: Var[
            list[Literal["description", "error", "input", "label"]]
        ]
        | list[Literal["description", "error", "input", "label"]]
        | None = None,
        label: Component | Var[Component | str] | str | None = None,
        label_props: Var[dict] | dict | None = None,
        limit: Var[int] | int | None = None,
        max_dropdown_height: Var[float | str] | float | str | None = None,
        max_tags: Var[int] | int | None = None,
        pointer: Var[bool] | bool | None = None,
        radius: Literal["lg", "md", "sm", "xl", "xs"]
        | Var[Literal["lg", "md", "sm", "xl", "xs"] | int | str]
        | int
        | str
        | None = None,
        render_option: Callable | Var[Callable] | None = None,
        required: Var[bool] | bool | None = None,
        scroll_area_props: Var[dict] | dict | None = None,
        search_value: Var[str] | str | None = None,
        select_first_option_on_change: Var[bool] | bool | None = None,
        size: Literal["lg", "md", "sm", "xl", "xs"]
        | Var[Literal["lg", "md", "sm", "xl", "xs"]]
        | None = None,
        split_chars: Var[list[str]] | list[str] | None = None,
        value: Var[list[str]] | list[str] | None = None,
        with_asterisk: Var[bool] | bool | None = None,
        with_error_styles: Var[bool] | bool | None = None,
        with_scroll_area: Var[bool] | bool | None = None,
        clearable: Var[bool] | bool | None = None,
        placeholder: Var[str] | str | None = None,
        multiline: Var[bool] | bool | None = None,
        read_only: Var[bool] | bool | None = None,
        style: Sequence[Mapping[str, Any]]
        | Mapping[str, Any]
        | Var[Mapping[str, Any]]
        | Breakpoints
        | None = None,
        key: Any | None = None,
        id: Any | None = None,
        ref: Var | None = None,
        class_name: Any | None = None,
        custom_attrs: dict[str, Var | Any] | None = None,
        left_section: Component | Var[Component | str] | str | None = None,
        left_section_props: Var[dict] | dict | None = None,
        left_section_width: Var[float | str] | float | str | None = None,
        left_section_pointer_events: Var[str] | str | None = None,
        right_section: Component | Var[Component | str] | str | None = None,
        right_section_props: Var[dict] | dict | None = None,
        right_section_width: Var[float | str] | float | str | None = None,
        right_section_pointer_events: Var[str] | str | None = None,
        on_blur: Optional[EventType[()]] = None,
        on_change: Optional[EventType[()] | EventType[list[str]]] = None,
        on_clear: Optional[EventType[Any]] = None,
        on_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_context_menu: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_double_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_dropdown_close: Optional[EventType[Any]] = None,
        on_dropdown_open: Optional[EventType[Any]] = None,
        on_duplicate: Optional[EventType[()] | EventType[str]] = None,
        on_focus: Optional[EventType[()]] = None,
        on_mount: Optional[EventType[()]] = None,
        on_mouse_down: Optional[EventType[()]] = None,
        on_mouse_enter: Optional[EventType[()]] = None,
        on_mouse_leave: Optional[EventType[()]] = None,
        on_mouse_move: Optional[EventType[()]] = None,
        on_mouse_out: Optional[EventType[()]] = None,
        on_mouse_over: Optional[EventType[()]] = None,
        on_mouse_up: Optional[EventType[()]] = None,
        on_option_submit: Optional[EventType[()] | EventType[str]] = None,
        on_remove: Optional[EventType[()] | EventType[str]] = None,
        on_scroll: Optional[EventType[()]] = None,
        on_scroll_end: Optional[EventType[()]] = None,
        on_search_change: Optional[EventType[Any]] = None,
        on_unmount: Optional[EventType[()]] = None,
        **props,
    ) -> "TagsInput":
        """Create the Reflex Enterprise component."""
        ...

tags_input = TagsInput.create
