"""Popup component for displaying information on a map."""

from .base import BaseLeafletComponent


class Popup(BaseLeafletComponent):
    """Popup component for displaying information on a map."""

    tag = "Popup"
    ...
